@@ -165,6 +165,7 @@
 -- See "Data.SBV.TP" for the API, and
 --
 --    - "Documentation.SBV.Examples.TP.BinarySearch"
+--    - "Documentation.SBV.Examples.TP.GCD"
 --    - "Documentation.SBV.Examples.TP.InsertionSort"
 --    - "Documentation.SBV.Examples.TP.MergeSort"
 --    - "Documentation.SBV.Examples.TP.QuickSort"
@@ -226,7 +227,7 @@ module Data.SBV (
   , SFPQuad, FPQuad
   , fpFromInteger
   -- ** Rationals
-  , SRational
+  , SRational, (.%)
   -- ** Algebraic reals
   -- $algReals
   , SReal, AlgReal(..), sRealToSInteger, algRealToRational, RealPoint(..), realPoint, RationalCV(..)
@@ -548,6 +549,8 @@ import Data.Char (isSpace, isPunctuation)
 import Data.SBV.List (EnumSymbolic(..))
 import Data.SBV.SEnum (sEnum)
 
+import Data.SBV.Rational
+
 #ifdef DOCTEST
 --- $setup
 --- >>> :set -XDataKinds -XFlexibleContexts -XTypeApplications -XRankNTypes
@@ -1148,6 +1151,7 @@ options turned on:
 >   LANGUAGE StandaloneDeriving
 >   LANGUAGE DeriveDataTypeable
 >   LANGUAGE DeriveAnyClass
+>   LANGUAGE FlexibleInstances
 
 and your own declaration must have instances of 'Enum' and 'Bounded'. (The instances can be derived, as above.)
 This will automatically introduce the type:
