@@ -40,6 +40,9 @@ module Data.SBV.List (
         -- * Containment
         , elem, notElem, isInfixOf, isSuffixOf, isPrefixOf
 
+        -- * List equality
+        , listEq
+
         -- * Sublists
         , take, drop, splitAt, subList, replace, indexOf, offsetIndexOf
 
@@ -239,7 +242,7 @@ singleton = lift1 False (SeqUnit (kindOf (Proxy @a))) (Just (: []))
 -- Q.E.D.
 -- >>> sat $ \(l :: SList Word16) -> length l .>= 2 .&& listToListAt l 0 ./= listToListAt l (length l - 1)
 -- Satisfiable. Model:
---   s0 = [0,64] :: [Word16]
+--   s0 = [0,32] :: [Word16]
 listToListAt :: SymVal a => SList a -> SInteger -> SList a
 listToListAt s offset = subList s offset 1
 
@@ -370,6 +373,16 @@ pre `isPrefixOf` l
   | True
   = lift2 True (SeqPrefixOf (kindOf (Proxy @a))) (Just L.isPrefixOf) pre l
 
+-- | @listEq@ is a variant of equality that you can use for lists of floats. It respects @NaN /= NaN@. The reason
+-- we do not do this automatically is that it complicates proof objectives usually, as it does not simply resolve to
+-- the native equality check.
+listEq :: forall a. SymVal a => SList a -> SList a -> SBool
+listEq
+  | containsFloats (kindOf (Proxy @a))
+  = smtFunction "listEq" $ \xs ys -> ite (null xs) (null ys) (head xs .== head ys .&& listEq (tail xs) (tail ys))
+  | True
+  = (.==)
+
 -- | @`isSuffixOf` suf l@. Is @suf@ a suffix of @l@?
 --
 -- >>> prove $ \(l1 :: SList Word16) l2 -> l2 `isSuffixOf` (l1 ++ l2)
