@@ -388,41 +388,6 @@ declRationals = [ "(declare-datatype SBVRational ((SBV.Rational (sbv.rat.numerat
                 , "(define-fun sbv.rat.notEq ((x SBVRational) (y SBVRational)) Bool"
                 , "   (not (sbv.rat.eq x y))"
                 , ")"
-                , ""
-                , "(define-fun sbv.rat.lt ((x SBVRational) (y SBVRational)) Bool"
-                , "   (<  (* (sbv.rat.numerator   x) (sbv.rat.denominator y))"
-                , "       (* (sbv.rat.denominator x) (sbv.rat.numerator   y)))"
-                , ")"
-                , ""
-                , "(define-fun sbv.rat.leq ((x SBVRational) (y SBVRational)) Bool"
-                , "   (<= (* (sbv.rat.numerator   x) (sbv.rat.denominator y))"
-                , "       (* (sbv.rat.denominator x) (sbv.rat.numerator   y)))"
-                , ")"
-                , ""
-                , "(define-fun sbv.rat.plus ((x SBVRational) (y SBVRational)) SBVRational"
-                , "   (SBV.Rational (+ (* (sbv.rat.numerator   x) (sbv.rat.denominator y))"
-                , "                    (* (sbv.rat.denominator x) (sbv.rat.numerator   y)))"
-                , "                 (* (sbv.rat.denominator x) (sbv.rat.denominator y)))"
-                , ")"
-                , ""
-                , "(define-fun sbv.rat.minus ((x SBVRational) (y SBVRational)) SBVRational"
-                , "   (SBV.Rational (- (* (sbv.rat.numerator   x) (sbv.rat.denominator y))"
-                , "                    (* (sbv.rat.denominator x) (sbv.rat.numerator   y)))"
-                , "                 (* (sbv.rat.denominator x) (sbv.rat.denominator y)))"
-                , ")"
-                , ""
-                , "(define-fun sbv.rat.times ((x SBVRational) (y SBVRational)) SBVRational"
-                , "   (SBV.Rational (* (sbv.rat.numerator   x) (sbv.rat.numerator y))"
-                , "                 (* (sbv.rat.denominator x) (sbv.rat.denominator y)))"
-                , ")"
-                , ""
-                , "(define-fun sbv.rat.uneg ((x SBVRational)) SBVRational"
-                , "   (SBV.Rational (* (- 1) (sbv.rat.numerator x)) (sbv.rat.denominator x))"
-                , ")"
-                , ""
-                , "(define-fun sbv.rat.abs ((x SBVRational)) SBVRational"
-                , "   (SBV.Rational (abs (sbv.rat.numerator x)) (sbv.rat.denominator x))"
-                , ")"
                 ]
 
 -- | Convert in a query context.
@@ -1027,7 +992,8 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
                                 , (Abs,           liftAbs)
                                 , (Quot,          lift2S  "bvudiv" "bvsdiv")
                                 , (Rem,           lift2S  "bvurem" "bvsrem")
-                                , (Equal,         eqBV)
+                                , (Equal True,    eqBV)
+                                , (Equal False,   eqBV)
                                 , (NotEqual,      neqBV)
                                 , (LessThan,      lift2S  "bvult" "bvslt")
                                 , (GreaterThan,   lift2S  "bvugt" "bvsgt")
@@ -1065,7 +1031,8 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
                                     , (Times,         lift2WM "*" "fp.mul")
                                     , (UNeg,          lift1FP "-" "fp.neg")
                                     , (Abs,           liftAbs)
-                                    , (Equal,         equal)
+                                    , (Equal True,    equal)
+                                    , (Equal False,   equal)
                                     , (NotEqual,      notEqual)
                                     , (LessThan,      lift2Cmp  "<"  "fp.lt")
                                     , (GreaterThan,   lift2Cmp  ">"  "fp.gt")
@@ -1073,27 +1040,16 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
                                     , (GreaterEq,     lift2Cmp  ">=" "fp.geq")
                                     ]
 
-                ratOpTable = [ (Plus,        lift2Rat "sbv.rat.plus")
-                             , (Minus,       lift2Rat "sbv.rat.minus")
-                             , (Times,       lift2Rat "sbv.rat.times")
-                             , (UNeg,        liftRat  "sbv.rat.uneg")
-                             , (Abs,         liftRat  "sbv.rat.abs")
-                             , (Equal,       lift2Rat "sbv.rat.eq")
+                ratOpTable = [ (Equal True,  lift2Rat "sbv.rat.eq")
+                             , (Equal False, lift2Rat "sbv.rat.eq")
                              , (NotEqual,    lift2Rat "sbv.rat.notEq")
-                             , (LessThan,    lift2Rat "sbv.rat.lt")
-                             , (GreaterThan, lift2Rat "sbv.rat.lt" . swap)
-                             , (LessEq,      lift2Rat "sbv.rat.leq")
-                             , (GreaterEq,   lift2Rat "sbv.rat.leq" . swap)
                              ]
                         where lift2Rat o [x, y] = "(" ++ o ++ " " ++ x ++ " " ++ y ++ ")"
                               lift2Rat o sbvs   = error $ "SBV.SMTLib2.sh.lift2Rat: Unexpected arguments: "   ++ show (o, sbvs)
-                              liftRat  o [x]    = "(" ++ o ++ " " ++ x ++ ")"
-                              liftRat  o sbvs   = error $ "SBV.SMTLib2.sh.lift2Rat: Unexpected arguments: "   ++ show (o, sbvs)
-                              swap [x, y]       = [y, x]
-                              swap sbvs         = error $ "SBV.SMTLib2.sh.swap: Unexpected arguments: "   ++ show sbvs
 
                 -- equality and comparisons are the only thing that works on uninterpreted sorts and pretty much everything else
-                uninterpretedTable = [ (Equal,       lift2S "="        "="        True)
+                uninterpretedTable = [ (Equal True,  lift2S "="        "="        True)
+                                     , (Equal False, lift2S "="        "="        True)
                                      , (NotEqual,    liftNS "distinct" "distinct" True)
                                      , (LessThan,    unintComp "<")
                                      , (GreaterThan, unintComp ">")
@@ -1102,7 +1058,8 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
                                      ]
 
                 -- For strings, equality and comparisons are the only operators
-                smtStringTable = [ (Equal,       lift2S "="        "="        True)
+                smtStringTable = [ (Equal True,  lift2S "="        "="        True)
+                                 , (Equal False, lift2S "="        "="        True)
                                  , (NotEqual,    liftNS "distinct" "distinct" True)
                                  , (LessThan,    stringCmp False "str.<")
                                  , (GreaterThan, stringCmp True  "str.<")
@@ -1110,9 +1067,9 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
                                  , (GreaterEq,   stringCmp True  "str.<=")
                                  ]
 
-                -- For lists, equality is really the only operator
+                -- For lists, equality is really the only operator. Also, not strong-equality due to lists of floats.
                 -- Likewise here, things might change for comparisons
-                smtListTable = [ (Equal,       lift2S "="        "="        True)
+                smtListTable = [ (Equal False, lift2S "="        "="        True)
                                , (NotEqual,    liftNS "distinct" "distinct" True)
                                , (LessThan,    seqCmp False "seq.<")
                                , (GreaterThan, seqCmp True  "seq.<")
