@@ -173,9 +173,10 @@ tailsLength =
 -- This property comes from Richard Bird's "Pearls of functional Algorithm Design" book, chapter 2.
 -- Note that it is not exactly as stated there, as the definition of @tail@ Bird uses is different
 -- than the standard Haskell function @tails@: Bird's version does not return the empty list as the
--- tail. So, we slightly modify it to fit the standard definition.
+-- tail. So, we slightly modify it to fit the standard definition. (NB. z3 is finicky on this
+-- problem, while cvc5 works much better.)
 --
--- >>> runTP $ tailsAppend @Integer
+-- >>> runTPWith cvc5 $ tailsAppend @Integer
 -- Inductive lemma: base case
 --   Step: Base                            Q.E.D.
 --   Step: 1                               Q.E.D.
@@ -611,6 +612,7 @@ mapReverse f = do
 --   Step: 2                               Q.E.D.
 --   Step: 3                               Q.E.D.
 --   Step: 4                               Q.E.D.
+--   Step: 5                               Q.E.D.
 --   Result:                               Q.E.D.
 -- [Proven] mapCompose :: Ɐxs ∷ [Integer] → Bool
 mapCompose :: forall a b c. (SymVal a, SymVal b, SymVal c) => (SBV a -> SBV b) -> (SBV b -> SBV c) -> TP (Proof (Forall "xs" [a] -> SBool))
@@ -621,6 +623,7 @@ mapCompose f g =
                            =: map g (f x .: map f xs)
                            =: g (f x) .: map g (map f xs)
                            ?? ih
+                           =: g (f x) .: map (g . f) xs
                            =: (g . f) x .: map (g . f) xs
                            =: map (g . f) (x .: xs)
                            =: qed
@@ -864,7 +867,7 @@ foldrFoldlDualityGeneralized e (@) = do
 --   Step: 5                               Q.E.D.
 --   Result:                               Q.E.D.
 -- [Proven] foldrFoldl :: Ɐxs ∷ [Integer] → Bool
-foldrFoldl :: forall a b. (SymVal a, SymVal b) => (SBV a -> SBV b -> SBV b) -> (SBV b -> SBV a -> SBV b) -> SBV b-> TP (Proof (Forall "xs" [a] -> SBool))
+foldrFoldl :: forall a b. (SymVal a, SymVal b) => (SBV a -> SBV b -> SBV b) -> (SBV b -> SBV a -> SBV b) -> SBV b -> TP (Proof (Forall "xs" [a] -> SBool))
 foldrFoldl (<+>) (<*>) e = do
    -- Assumptions about the operators
    let -- (x <+> y) <*> z == x <+> (y <*> z)
@@ -1380,7 +1383,7 @@ take_cons = lemma "take_cons"
 
 -- | @take n (map f xs) == map f (take n xs)@
 --
--- >>> runTP $ take_map @Integer @Float (uninterpret "f")
+-- >>> runTP $ take_map @Integer @Integer (uninterpret "f")
 -- Lemma: take_cons                        Q.E.D.
 -- Lemma: map1                             Q.E.D.
 -- Lemma: take_map.n <= 0                  Q.E.D.
@@ -1392,7 +1395,9 @@ take_cons = lemma "take_cons"
 --   Step: 4                               Q.E.D.
 --   Step: 5                               Q.E.D.
 --   Result:                               Q.E.D.
--- Lemma: take_map                         Q.E.D.
+-- Lemma: take_map
+--   Step: 1                               Q.E.D.
+--   Result:                               Q.E.D.
 -- [Proven] take_map :: Ɐn ∷ Integer → Ɐxs ∷ [Integer] → Bool
 take_map :: forall a b. (SymVal a, SymVal b) => (SBV a -> SBV b) -> TP (Proof (Forall "n" Integer -> Forall "xs" [a] -> SBool))
 take_map f = do
@@ -1419,9 +1424,13 @@ take_map f = do
                                            =: map f (take n (x .: xs))
                                            =: qed
 
-    lemma "take_map"
-          (\(Forall n) (Forall xs) -> take n (map f xs) .== map f (take n xs))
-          [proofOf h1, proofOf h2]
+    calc "take_map"
+         (\(Forall n) (Forall xs) -> take n (map f xs) .== map f (take n xs)) $
+         \n xs -> [] |- take n (map f xs)
+                     ?? h1
+                     ?? h2
+                     =: map f (take n xs)
+                     =: qed
 
 -- | @n .> 0 ==> drop n (x .: xs) == drop (n - 1) xs@
 --
