@@ -32,10 +32,11 @@ module Data.SBV.TP.TP (
        ,  induct,  inductWith
        , sInduct, sInductWith
        , sorry
-       , TP, runTP, runTPWith, tpQuiet, tpRibbon, tpStats, tpCache
-       , (|-), (⊢), (=:), (≡), (??), (∵), split, split2, cases, (==>), (⟹), qed, trivial, contradiction
+       , TP, runTP, runTPWith, tpQuiet, tpRibbon, tpStats, tpAsms, tpCache
+       , (|-), (|->), (⊢), (=:), (≡), (??), (∵), split, split2, cases, (==>), (⟹), qed, trivial, contradiction
        , qc, qcWith
        , disp
+       , recall
        ) where
 
 import Data.SBV
@@ -44,7 +45,7 @@ import Data.SBV.Core.Data  (SBV(..), SVal(..))
 import qualified Data.SBV.Core.Symbolic as S (sObserve)
 
 import Data.SBV.Core.Operations (svEqual)
-import Data.SBV.Control hiding (getProof)
+import Data.SBV.Control hiding (getProof, (|->))
 
 import Data.SBV.TP.Kernel
 import Data.SBV.TP.Utils
@@ -242,7 +243,7 @@ proveProofTree cfg tpSt nm (result, resultBool) initialHypotheses calcProofTree
                                               , isCached     = False
                                               }
 
-  where SMTConfig{tpOptions = TPOptions{printStats}} = cfg
+  where SMTConfig{tpOptions = TPOptions{printStats, printAsms}} = cfg
 
         isEnd ProofEnd{}    = True
         isEnd ProofStep{}   = False
@@ -314,8 +315,8 @@ proveProofTree cfg tpSt nm (result, resultBool) initialHypotheses calcProofTree
 
                  -- First prove the assumptions, if there are any. We stay quiet, unless timing is asked for
                  (quietCfg, finalizer)
-                   | printStats = (cfg,                                             finish [] [])
-                   | True       = (cfg{tpOptions = (tpOptions cfg) {quiet = True}}, const (pure ()))
+                   | printStats || printAsms = (cfg,                                             finish [] [])
+                   | True                    = (cfg{tpOptions = (tpOptions cfg) {quiet = True}}, const (pure ()))
 
                  as = concatMap getHelperAssumes hs
                  ss = getHelperText hs
@@ -390,8 +391,8 @@ data CalcContext a = CalcStart     [Helper] -- Haven't started yet
 
 
 -- | Turn a raw (i.e., as written by the user) proof tree to a tree where the successive equalities are made explicit.
-mkProofTree :: SymVal a => (SBV a -> SBV a -> c) -> TPProofRaw (SBV a) -> TPProofGen c [String] SBool
-mkProofTree symEq = go (CalcStart [])
+mkProofTree :: SymVal a => (SBV a -> SBV a -> c, SBV a -> SBV a -> SBool) -> TPProofRaw (SBV a) -> TPProofGen c [String] SBool
+mkProofTree (symTraceEq, symEq) = go (CalcStart [])
   where -- End of the proof; tie the begin and end
         go step (ProofEnd () hs) = case step of
                                      -- It's tempting to error out if we're at the start and already reached the end
@@ -399,7 +400,7 @@ mkProofTree symEq = go (CalcStart [])
                                      -- general case, it's quite valid in a case-split; where one of the case-splits
                                      -- might be easy enough for the solver to deduce so the user simply says "just derive it for me."
                                      CalcStart hs'           -> ProofEnd sTrue (hs' ++ hs) -- Nothing proven!
-                                     CalcStep  begin end hs' -> ProofEnd (begin .== end) (hs' ++ hs)
+                                     CalcStep  begin end hs' -> ProofEnd (begin `symEq` end) (hs' ++ hs)
 
         -- Branch: Just push it down. We use the hints from previous step, and pass the current ones down.
         go step (ProofBranch c hs ps) = ProofBranch c (getHelperText hs) [(bc, go step' p) | (bc, p) <- ps]
@@ -409,20 +410,20 @@ mkProofTree symEq = go (CalcStart [])
 
         -- Step:
         go (CalcStart hs)           (ProofStep cur hs' p) = go (CalcStep cur cur (hs' ++ hs)) p
-        go (CalcStep first prev hs) (ProofStep cur hs' p) = ProofStep (symEq prev cur) hs (go (CalcStep first cur hs') p)
+        go (CalcStep first prev hs) (ProofStep cur hs' p) = ProofStep (prev `symTraceEq` cur) hs (go (CalcStep first cur hs') p)
 
 -- | Turn a sequence of steps into a chain of equalities
 mkCalcSteps :: SymVal a => (SBool, TPProofRaw (SBV a)) -> ([Int] -> Symbolic SBool) -> Symbolic CalcStrategy
 mkCalcSteps (intros, tpp) qcInstance = do
         pure $ CalcStrategy { calcIntros     = intros
-                            , calcProofTree  = mkProofTree (.==) tpp
+                            , calcProofTree  = mkProofTree ((.===), (.===)) tpp
                             , calcQCInstance = qcInstance
                             }
 
 -- | Given initial hypothesis, and a raw proof tree, build the quick-check walk over this tree for the step that's marked as such.
 qcRun :: SymVal a => [Int] -> (SBool, TPProofRaw (SBV a)) -> Symbolic SBool
 qcRun checkedLabel (intros, tpp) = do
-        results <- runTree sTrue 1 ([1], mkProofTree (\a b -> (a, b, a .== b)) tpp)
+        results <- runTree sTrue 1 ([1], mkProofTree (\a b -> (a, b, a .=== b), (.==)) tpp)
         case [b | (l, b) <- results, l == checkedLabel] of
           [(caseCond, b)] -> do constrain $ intros .&& caseCond
                                 pure b
@@ -1443,6 +1444,17 @@ instance Contradiction a => Contradiction (b -> a) where
 bs |- p = (sAnd bs, p)
 infixl 0 |-
 
+-- | Start an implicational  proof, with the given hypothesis. Use @[]@ as the
+-- first argument if the calculation holds unconditionally. Each step will be a cascading
+-- chain of conjunctions of the previous, starting from @sTrue@.
+(|->) :: [SBool] -> TPProofRaw SBool -> (SBool, TPProofRaw SBool)
+bs |-> p = (sAnd bs, xform sTrue p)
+  where xform :: SBool -> TPProofGen SBool [Helper] () -> TPProofGen SBool [Helper] ()
+        xform conj (ProofStep   a hs r)  = let ca = conj .&& a in ProofStep ca hs (xform ca r)
+        xform conj (ProofBranch b bh ss) = ProofBranch b bh [(bc, xform conj r) | (bc, r) <- ss]
+        xform _    (ProofEnd    b hs )   = ProofEnd b hs
+infixl 0 |->
+
 -- | Alternative unicode for `|-`.
 (⊢) :: [SBool] -> TPProofRaw a -> (SBool, TPProofRaw a)
 (⊢) = (|-)
@@ -1457,7 +1469,7 @@ split :: SymVal a => SList a -> TPProofRaw r -> (SBV a -> SList a -> TPProofRaw
 split xs empty cons = ProofBranch False [] [(cnil, empty), (ccons, cons h t)]
    where cnil   = SL.null   xs
          (h, t) = SL.uncons xs
-         ccons  = sNot cnil .&& xs .== h SL..: t
+         ccons  = sNot cnil .&& xs .=== h SL..: t
 
 -- | Case splitting over two lists; empty and full cases for each
 split2 :: (SymVal a, SymVal b)
@@ -1476,11 +1488,11 @@ split2 (xs, ys) ee ec ce cc = ProofBranch False
                                           ]
   where xnil     = SL.null   xs
         (hx, tx) = SL.uncons xs
-        xcons    = sNot xnil .&& xs .== hx SL..: tx
+        xcons    = sNot xnil .&& xs .=== hx SL..: tx
 
         ynil     = SL.null   ys
         (hy, ty) = SL.uncons ys
-        ycons    = sNot ynil .&& ys .== hy SL..: ty
+        ycons    = sNot ynil .&& ys .=== hy SL..: ty
 
 -- | A quick-check step, taking number of tests.
 qc :: Int -> Helper
@@ -1504,4 +1516,18 @@ infix 0 ==>
 (⟹) = (==>)
 infix 0 ⟹
 
+-- | Recalling a proof. This essentially sets the verbose output off during this proof. Note that
+-- if we're doing stats, we ignore this as the whole point of doing stats is to see steps in detail.
+recall :: String -> TP (Proof a) -> TP (Proof a)
+recall nm prf = do
+  cfg <- getTPConfig
+  if printStats (tpOptions cfg)
+     then prf
+     else do tab <- liftIO $ startTP cfg (verbose cfg) "Lemma" 0 (TPProofOneShot nm [])
+             setTPConfig cfg{tpOptions = (tpOptions cfg) {quiet = True}}
+             r@Proof{proofOf = ProofObj{dependencies}} <- prf
+             setTPConfig cfg
+             liftIO $ finishTP cfg ("Q.E.D." ++ concludeModulo dependencies) (tab, Nothing) []
+             pure r
+
 {- HLint ignore module "Eta reduce" -}
